#include <rofi/module.hpp>
namespace rofi
{

void Module::makeNodeTree()
{
    for (auto &voxel: parts->getVoxels())
        node->addChild(voxel->getNode());

    for (auto &connector : parts->getConnections())
        node->addChild(connector->getNode());
}

void Module::setConnectorDocks() // TO DO: change if connector placement gets changed
{
    if (type == "cube")
        setConnectorDocksCube();
    if (type == "universal")
        setConnectorDocksUniversal();
}

void Module::setConnectorDocksCube()
{

    // if (component_ids.empty())
    //     setComponentIDs();

    // auto body_node = parts->getVoxels().front()->getNode();
    // body_node->rotateRotationQuat(glm::angleAxis(glm::radians(90.0f), glm::vec3(0.0f, -1.0f, 0.0f)));

    // using enum side;
    // for (const auto &connector : parts->getConnections())
    // {
    //     const auto roficom_id = component_ids.at(connector);
    //     auto con_node = connector->getNode();
    //     glm::quat rotation;
    //     switch(roficom_id)
    //     {
    //         case 0:
    //             rotation = (glm::angleAxis(glm::radians(90.0f), enum_to_vec(z_pos)));
    //             break;
    //         case 1:
    //             rotation = (glm::angleAxis(glm::radians(90.0f), enum_to_vec(z_neg)));
    //             break;
    //         case 3:
    //             rotation = (glm::angleAxis(glm::radians(180.0f), enum_to_vec(z_pos)));
    //             break;
    //         case 2:
    //         case 4:
    //         case 5:
    //             rotation = glm::quat(1.0f, 0.0f, 0.0f, 0.0f);
    //             break;
    //     }

    //     con_node->rotateRotationQuat(rotation);
    // }
}

void Module::setConnectorDocksUniversal()
{
    using enum side;
    std::string dock;
    auto a_pos = glm::vec3(0.0f);

    for (const auto &connector : parts->getConnections())
    {
        if (connector->getType(false) == connector_type::shared_rotation)
            continue;

        auto con_side = connector->getVoxelSide().first;
        auto voxel_pos = connector->getConnection().first.lock()->getNode()->getPositionLocal();
        switch(con_side)
        {
            case x_neg:
                dock = voxel_pos == a_pos ? "A+X" : "B+X";
                break;
            case x_pos:
                dock = voxel_pos == a_pos ? "A-X" : "B-X";
                break;
            case z_neg:
                dock = voxel_pos == a_pos ? "A-Z" : "B-Z";
                break;
            default:
                ASSUMPTION(false);
        }
        connector_docks.emplace(connector, dock);
    }
}

void Module::setComponentIDs()
{
    component_ids.clear();

    if (type == "universal")
        setComponentIDsUniversal();
    else
        setComponentIDsGeneric();
}

void Module::setComponentIDsUniversal()
{
    ASSUMPTION(connector_docks.size() == 6);

    // RoFICoMs
    for (const auto &[connector, dock] : connector_docks)
    {
        if (dock == "A-X")
            component_ids.emplace(connector, 0);
        else if (dock == "A+X")
            component_ids.emplace(connector, 1);
        else if (dock == "A-Z")
            component_ids.emplace(connector, 2);
        else if (dock == "B-X")
            component_ids.emplace(connector, 3);
        else if (dock == "B+X")
            component_ids.emplace(connector, 4);
        else if (dock == "B-Z")
            component_ids.emplace(connector, 5);
    }

    // Bodies
    auto a_pos = glm::vec3(0.0f);
    voxel_ptr a_voxel;

    for (const auto &voxel : parts->getVoxels())
    {
        if (voxel->getNode()->getPositionLocal() == a_pos)
        {
            component_ids.emplace(voxel, 8);
            a_voxel = voxel;
        }
        else
            component_ids.emplace(voxel, 9);
    }

    // Shoes // Shoes do not get placed into the map as the key - connector is already placed as roficom
    // for (const auto &connector : parts->getConnections())
    // {
    //     if (auto rot_con = std::dynamic_pointer_cast<RotationConnector>(connector);
    //         rot_con && rot_con->isHead())
    //     {
    //         component_ids.emplace(connector, connector->getConnection().first.lock() == a_voxel ? 6 : 7);
    //     }
    // }

}

void Module::setComponentIDsGeneric()
{
    for (const auto connector : parts->getConnections())
    {
        if (connector->getType(true) == connector_type::open)
            component_ids.emplace(connector, component_ids.size());
    }
    for (const auto connector : parts->getConnections())
    {
        if (connector->getType(true) == connector_type::rotation)
            component_ids.emplace(connector, component_ids.size());
    }

    for (const auto voxel : parts->getVoxels())
    {
        if (!std::dynamic_pointer_cast<ModuleLink>(voxel) && !std::dynamic_pointer_cast<PadBoard>(voxel))
            component_ids.emplace(voxel, component_ids.size());
    }
}

Module::Module(const Module &other)
{
    node = Node::copy_deep(other.node);
    parts = VoxelGraph::copy(other.parts, node->getChildren());
    name = other.name;
    type = other.type;
    setConnectorDocks();
    setComponentIDs();
}
Module::Module(const Module &&other)
{
    node = std::move(other.node);
    parts = std::move(other.parts);
    name = std::move(other.name);
    type = std::move(other.type);
    setConnectorDocks();
    setComponentIDs();
}
Module& Module::operator=(const Module &other)
{
    node = Node::copy_deep(other.node);
    parts = VoxelGraph::copy(other.parts, node->getChildren());
    name = other.name;
    type = other.type;
    component_ids = other.component_ids;
    setConnectorDocks();
    setComponentIDs();

    return *this;
}
Module& Module::operator=(const Module &&other)
{
    node = std::move(other.node);
    parts = std::move(other.parts);
    name = std::move(other.name);
    type = std::move(other.type);
    setConnectorDocks();
    setComponentIDs();

    return *this;
}

Module::Module(voxel_graph_ptr _parts, std::string &_name, std::string &_type, std::map<objectbase_ptr, uint64_t> &_component_ids)
: node{Node::create()}, parts{_parts}, name{_name}, type{_type}, component_ids{_component_ids}
{
    ASSUMPTION(!_parts->getVoxels().empty() || !_parts->getConnections().empty());
    makeNodeTree();
    setConnectorDocks();
    setComponentIDs();
}

Module::Module(voxel_graph_ptr _parts, std::string &&_name, std::string &&_type, std::map<objectbase_ptr, uint64_t> &&_component_ids)
: node{Node::create()}, parts{_parts}, name{std::move(_name)}, type{std::move(_type)}, component_ids{std::move(_component_ids)}
{
    ASSUMPTION(!_parts->getVoxels().empty() || !_parts->getConnections().empty());
    makeNodeTree();
    setConnectorDocks();
    setComponentIDs();
}

module_ptr Module::copy(const module_ptr other_module)
{
    return module_ptr{ new Module(*other_module) };
}

}
